require_relative './config.rb'
require_relative './errors/parser.rb'
require_relative './base.rb'

Struct.new('Config', :rabbit_mq)
Struct.new('RabbitMq', :connection_string, :queue_names)
Struct.new('QueueName', :publish, :subscribe, :durable, :auto_delete, :exchange_name, :exclusive)

module Services
  module Config
    class ConfigStruct
      include Services::Config::Base::ConfigStrucBase
      attr_reader :config, :queue_names, :rabbit_mq

      def initialize(config_hash = nil)
        setup_vars(config_hash)
      end

      def setup_vars(config_hash)
        if !config_hash.nil?
          @is_basic = config_hash.fetch(BASIC_KEY, false)
          @config_hash = config_hash
          @config = Struct::Config.new(config_hash[ROOT_KEY])
          @queue_names = config_hash[ROOT_KEY][QUEUE_NAME_KEY].map do |val|
            params = QUEUE_NAME_KEYS.map { |key| val.fetch(key, nil) }
            Struct::QueueName.new(*params)
          end
          @rabbit_mq = Struct::RabbitMq.new(config_hash[ROOT_KEY][CONNECTION_STRING_KEY], @queue_names)
        else
          throw Services::Config::Errors::InvalidConfig.new(e.message, INIT_ERR_MSG)
        end
      end

    end
  end
end
