require_relative './rabbit_wrapper.rb'

module Services
  module RabbitWrapper
    module BaseHelpers

      def basic_config_params(connection_string, exchange_name)
        {
          'rabbit_mq' => {
            'connection_string' => connection_string,
            'queue_names' => [
              {
                'exchange_name' => exchange_name,
                'durable' => true
              }
            ],
            'is_basic' => true
          }
        }
      end

      def setup_config(connection_string, exchange_name, config)
        if connection_string && exchange_name
          @is_basic = true
          @config = Services::Config::ConfigStruct.new(basic_config_params(connection_string, exchange_name))
        else
          @is_basic = false
          config.load_config
          @config = config.config_structs
        end
      end

      def sub_queue_name(queue_name = nil)
        return queue_name if @is_basic && !queue_name.nil?

        return 'queue_name.not.provided' if @is_basic && queue_name.nil?

        @config.sub_queue_name
      end

      def pub_queue_name(queue_name = nil)
        return queue_name if @is_basic && !queue_name.nil?

        return 'queue_name.not.provided' if @is_basic && queue_name.nil?

        @config.pub_queue_name
      end
    end
  end
end
